@@ -272,7 +272,7 @@ def toCSV(collection, filename, split_at=4000):
         return thecsv
 
 
-def toLocal(collection, filename, filetype=None, selectors=None):
+def toLocal(collection, filename, filetype=None, selectors=None, path=None):
     """ Download a FeatureCollection to a local file a CSV or geoJSON file.
     This uses a different method than `toGeoJSON` and `toCSV`
 
@@ -286,11 +286,11 @@ def toLocal(collection, filename, filetype=None, selectors=None):
         filetype = 'CSV'
 
     url = collection.getDownloadURL(filetype, selectors, filename)
-    thefile = utils.downloadFile(url, filename, filetype)
+    thefile = utils.downloadFile(url, filename, filetype, path)
     return thefile
 
 
-def toAsset(table, assetPath, name=None, create=True, **kwargs):
+def toAsset(table, assetPath, name=None, create=True, verbose=False, **kwargs):
     """ This function can create folders and ImageCollections on the fly.
     The rest is the same to Export.image.toAsset. You can pass the same
     params as the original function
@@ -320,9 +320,12 @@ def toAsset(table, assetPath, name=None, create=True, **kwargs):
     # Asset ID (Path + name)
     assetId = '/'.join([assetPath, name])
     # Description
-    description = name.replace('/','_')
+    description = utils.matchDescription(name)
     # Init task
-    task = ee.batch.Export.table.toAsset(table, assetId=assetId, description=description, **kwargs)
+    task = ee.batch.Export.table.toAsset(table, assetId=assetId,
+                                         description=description, **kwargs)
     task.start()
+    if verbose:
+        print('Exporting {} to {}'.format(name, assetPath))
 
     return task
\ No newline at end of file
