@@ -8,6 +8,7 @@ import ee.data
 import math
 from . import ee_list, date
 from ..utils import castImage
+from ..ui import map as mapui
 
 
 def _add_suffix_prefix(image, value, option, bands=None):
@@ -59,6 +60,13 @@ def addPrefix(image, prefix, bands=None):
     return _add_suffix_prefix(image, prefix, 'prefix', bands)
 
 
+def bufferMask(image, radius=1.5, kernelType='square', units='pixels'):
+    """ Make a buffer around the masked pixels """
+    masked = image.mask().Not()
+    buffer = masked.focal_max(radius, kernelType, units)
+    return image.updateMask(buffer.Not())
+
+
 def empty(value=0, names=None, from_dict=None):
     """ Create a constant image with the given band names and value, and/or
     from a dictionary of {name: value}
@@ -103,6 +111,19 @@ def emptyBackground(image, value=0):
         source=image, properties=image.propertyNames()))
 
 
+def emptyCopy(image, emptyValue=0, copyProperties=None, keepMask=False):
+    """ Make an empty copy of the given image """
+    footprint = image.geometry()
+    emp = empty(emptyValue, image.bandNames())
+    if copyProperties:
+        emp = emp.copyProperties(source=image, properties=copyProperties)
+    if keepMask:
+        emp = emp.updateMask(image.mask())
+
+    return ee.Image(ee.Algorithms.If(footprint.isUnbounded(),
+                                     emp, emp.clip(footprint)))
+
+
 def getValue(image, point, scale=None, side="server"):
     """ Return the value of all bands of the image in the specified point
 
@@ -859,6 +880,206 @@ def maskInside(image, geometry):
     return image.updateMask(mask)
 
 
+def paint(image, featurecollection, vis_params=None, color='black', width=1,
+          fillColor=None, **kwargs):
+    """ Paint a FeatureCollection onto an Image. Returns an Image with three
+    bands: vis-blue, vis-geen, vis-red (uint8)
+
+    It admits the same parameters as ee.FeatureCollection.style
+    """
+    if not fillColor:
+        fillColor = "#00000000"
+    if not vis_params:
+        firstband = ee.String(image.bandNames().get(0))
+        vis_params = dict(bands=[firstband, firstband, firstband], min=0,
+                          max=1)
+    region = image.geometry()
+    filtered = ee.FeatureCollection(
+        ee.Algorithms.If(
+            region.isUnbounded(),
+            featurecollection,
+            featurecollection.filterBounds(region)
+        ))
+    fcraster = filtered.style(color=color, width=width, fillColor=fillColor,
+                              **kwargs)
+    mask = fcraster.reduce('sum').gte(0).rename('mask')
+    topaint = image.visualize(**vis_params)
+    final = topaint.where(mask, fcraster)
+    final = final.copyProperties(source=image)
+    properties = image.propertyNames()
+    final = ee.Image(ee.Algorithms.If(
+        properties.contains('system:time_start'),
+        final.set('system:time_start', image.date().millis()),
+        final))
+    final = ee.Image(ee.Algorithms.If(
+        properties.contains('system:time_end'),
+        final.set('system:time_end', ee.Number(image.get('system:time_end'))),
+        final))
+
+    return final
+
+
+def repeatBand(image, times=None, names=None, properties=None):
+    """ Repeat one band. If the image parsed has more than one band, the first
+    will be used """
+    band = ee.Image(image.select([0]))
+    if times is not None:
+        times = ee.Number(times)
+        proxylist = ee.List.repeat(0, times.subtract(1))
+        def add(band, i):
+            band = ee.Image(band)
+            i = ee.Image(i)
+            return i.addBands(band)
+        proxyImg = proxylist.map(lambda n: band)
+        repeated = ee.Image(proxyImg.iterate(add, band))
+    else:
+        newNames = ee.List(names)
+        firstName = ee.String(newNames.get(0))
+        rest = ee.List(newNames.slice(1))
+        def add(name, i):
+            name = ee.String(name)
+            i = ee.Image(i)
+            return i.addBands(band.rename(name))
+        first = band.rename(firstName)
+        repeated = ee.Image(rest.iterate(add, first))
+
+    if properties:
+        repeated = repeated.setMulti(properties)
+
+    return ee.Image(repeated)
+
+
+def arrayNonZeros(image):
+    """
+    Return an image array without zeros
+
+    :param image:
+    :return:
+    """
+    def wrap(arr):
+        binarr = arr.divide(arr)
+        n = binarr.arrayReduce(ee.Reducer.sum(), [0]).multiply(-1)
+        nimg = n.arrayProject([0]).arrayFlatten([['n']]).toInt()
+        sorted = arr.arraySort()
+        sliced = sorted.arraySlice(0, nimg)
+        return sliced
+
+    bands = image.bandNames()
+    first = wrap(image.select([bands.get(0)]))
+    rest = bands.slice(1)
+
+    def overBands(band, i):
+        band = ee.String(band)
+        i = ee.Image(i)
+        array = wrap(image.select([band]))
+        return i.addBands(array)
+
+    result1 = ee.Image(rest.iterate(overBands, first))
+    result2 = ee.Image(wrap(image))
+
+    return ee.Image(ee.Algorithms.If(bands.size(), result1, result2))
+
+
+def getTileURL(image, visParams=None):
+    """ Get the URL for the given image passing a normal visualization
+    parameters like `{'bands':['B4','B3','B2'], 'min':0, 'max':5000}` """
+    if visParams:
+        vis = mapui.formatVisParams(visParams)
+        image_info = image.getMapId(vis)
+    else:
+        image_info = image.getMapId()
+    fetcher = image_info['tile_fetcher']
+    return fetcher.url_format
+
+
+def applyMask(image, mask, bands=None, negative=True):
+    """ Apply a passed positive mask """
+    bands = bands or mask.bandNames()
+    bands = ee.List(bands)
+    def wrap(band, img):
+        img = ee.Image(img)
+        band = ee.String(band)
+        m = mask.select(band)
+        toapply = m.Not() if negative else m
+        return img.updateMask(toapply)
+
+    return ee.Image(bands.iterate(wrap, image))
+
+
+def regionCover(image, region, bands=None, scale=None, operator='OR',
+                property_name='REGION_COVER', crs=None, crsTransform=None,
+                bestEffort=False, maxPixels=1e13, tileScale=1):
+    """ Compute the percentage of values greater than 1 in a region. If more
+     than one band is specified, it applies the specified operator """
+    operators = ['OR', 'AND']
+    if operator not in operators:
+        raise ValueError('operator must be one of {}'.format(operators))
+
+    if not bands:
+        bands = ee.List([image.bandNames().get(0)])
+
+    if operator == 'AND':
+        reducer = ee.Reducer.bitwiseAnd()
+    else:
+        reducer = ee.Reducer.bitwiseOr()
+
+    bandname = 'regionCover'
+    mask = image.select(bands).reduce(reducer).rename(bandname)
+
+    # get projection
+    projection = mask.projection()
+
+    if not scale:
+        scale = projection.nominalScale()
+
+    # Make an image with all ones
+    ones_i = ee.Image.constant(1).reproject(projection).rename(bandname)
+
+    # manage geometry types
+    if isinstance(region, (ee.Feature, ee.FeatureCollection)):
+        region = region.geometry()
+
+    unbounded = region.isUnbounded()
+
+    # Get total number of pixels
+    ones = ones_i.reduceRegion(
+        reducer= ee.Reducer.count(),
+        geometry= region,
+        scale= scale,
+        maxPixels= maxPixels,
+        crs=crs,
+        crsTransform=crsTransform,
+        bestEffort=bestEffort,
+        tileScale=tileScale,
+    ).get(bandname)
+    ones = ee.Number(ones)
+
+    # select first band, unmask and get the inverse
+    image_to_compute = mask.selfMask()
+
+    # Get number of zeros in the given image
+    zeros_in_mask =  image_to_compute.reduceRegion(
+        reducer= ee.Reducer.count(),
+        geometry= region,
+        scale= scale,
+        maxPixels= maxPixels,
+        crs=crs,
+        crsTransform=crsTransform,
+        bestEffort=bestEffort,
+        tileScale=tileScale
+    ).get(bandname)
+    zeros_in_mask = ee.Number(zeros_in_mask)
+
+    percentage = zeros_in_mask.divide(ones)
+
+    # Multiply by 100
+    cover = percentage.multiply(100)
+
+    # Return None if geometry is unbounded
+    final = ee.Number(ee.Algorithms.If(unbounded, 0, cover))
+    return image.set(property_name, final)
+
+
 class Classification(object):
     """ Class holding (static) methods for classified images. """
     @staticmethod
