@@ -35,6 +35,12 @@ def getId(collection):
     return collection.limit(0).getInfo()['id']
 
 
+def getImage(collection, index):
+    """ Get an Image using its collection index """
+    collist = collection.toList(collection.size())
+    return ee.Image(collist.get(index))
+
+
 def wrapper(f, *arg, **kwargs):
     """ Wrap a function and its arguments into a mapping function for
     ImageCollections. The first parameter of the functions must be an Image,
@@ -69,6 +75,23 @@ def enumerateProperty(collection, name='enumeration'):
     return ee.ImageCollection(imlist)
 
 
+def enumerateSimple(collection, name='ENUM'):
+    """ Simple enumeration of features inside a collection. Each feature stores
+     its enumeration, so if the order of features changes over time, the numbers
+     will not be in order """
+
+    size = collection.size()
+    collist = collection.toList(size)
+    seq = ee.List.sequence(0, size.subtract(1))
+    def wrap(n):
+        n = ee.Number(n).toInt()
+        feat = collist.get(n)
+        return ee.Image(feat).set(name, n)
+    fc = ee.ImageCollection.fromImages(seq.map(wrap))
+
+    return ee.ImageCollection(fc.copyProperties(source=collection))
+
+
 def fillWithLast(collection):
     """ Fill masked values of each image pixel with the last available
     value
@@ -86,7 +109,7 @@ def fillWithLast(collection):
         d = img.date()
         condition = d.difference(first_date, 'second')
         def true(date):
-            newcol = collection.filterDate(first_date, date)
+            newcol = collection.filterDate(first_date, date.advance(1, 'day'))
             return composite.closestDate(newcol)
         return ee.Image(ee.Algorithms.If(condition, true(d), img))
 
@@ -110,19 +133,16 @@ def mergeGeometries(collection):
     return ee.Geometry(rest.iterate(wrap, first.geometry()))
 
 
-def mosaicSameDay(collection, reducer=None):
+def mosaicSameDay(collection, qualityBand=None):
     """ Return a collection where images from the same day are mosaicked
 
-    :param reducer: the reducer to use for merging images from the same day.
-        Defaults to 'first'
-    :type reducer: ee.Reducer
+    :param qualityBand: the band that holds the quality score for mosaiking.
+        If None it will use the simplier mosaic() function
+    :type qualityBand: str
     :return: a new image collection with 1 image per day. The only property
         kept is `system:time_start`
     :rtype: ee.ImageCollection
     """
-    if reducer is None:
-        reducer = ee.Reducer.mean()
-
     def make_date_list(img, l):
         l = ee.List(l)
         img = ee.Image(img)
@@ -139,20 +159,38 @@ def mosaicSameDay(collection, reducer=None):
     col_list = collection.toList(collection.size())
     date_list = ee.List(col_list.iterate(make_date_list, ee.List([])))
 
+    first_img = ee.Image(collection.first())
+    bands = first_img.bandNames()
+
     def make_col(date):
         date = ee.Date(date)
-        def wrap(img):
-            e = image_module.emptyBackground(img, -9999)
-            return e.updateMask(e.neq(-9999))
+        filtered = collection.filterDate(date, date.advance(1, 'day'))
 
-        filtered = collection.filterDate(date, date.advance(1, 'day')).map(wrap)
-        first_img = ee.Image(collection.first())
+        if qualityBand:
+            mosaic = filtered.qualityMosaic(qualityBand)
+        else:
+            mosaic = filtered.mosaic()
 
-        mosaic = filtered.reduce(reducer).set(
-            'system:time_start', date.millis(),
-            'system:footprint', mergeGeometries(filtered)
-        )
-        return mosaic.rename(first_img.bandNames())
+        mosaic = mosaic.set('system:time_start', date.millis(),
+                            'system:footprint', mergeGeometries(filtered))
+
+        # mosaic = mosaic.rename(bands)
+        mosaic = mosaic.select(bands)
+        def reproject(bname, mos):
+            mos = ee.Image(mos)
+            mos_bnames = mos.bandNames()
+            bname = ee.String(bname)
+            proj = first_img.select(bname).projection()
+
+            newmos = ee.Image(ee.Algorithms.If(
+                mos_bnames.contains(bname),
+                image_module.replace(mos, bname, mos.select(bname).setDefaultProjection(proj)),
+                mos))
+
+            return newmos
+
+        mosaic = ee.Image(bands.iterate(reproject, mosaic))
+        return mosaic
 
     new_col = ee.ImageCollection.fromImages(date_list.map(make_col))
     return new_col
@@ -314,6 +352,54 @@ def getValues(collection, geometry, scale=None, reducer=None,
         raise ValueError("side parameter must be 'server' or 'client'")
 
 
+def outliers(collection, bands, sigma=2, updateMask=False):
+    """ Compute outliers by:
+
+    outlier = value > mean+(sigma*stddev)
+    outlier = value < mean-(sigma*stddev)
+
+    Example (sigma = 1):
+        - values = [1, 5, 6, 4, 7, 10]
+        - mean = 5.5
+        - std dev = 3
+        - mean + (sigma*stddev) = 8.5
+        - mean - (sigma*stddev) = 2.5
+        - outliers = values between 2.5 and 8.5 = [1, 10]
+
+    if `updateMask` is False return the passed collection in which each image
+    have new bands (a mask) corresponding to the passed dict and a suffix '_outlier'
+    else return the passed collection with the passed bands masked if are
+    outliers (the outlier band is not returned).
+
+    idea from: https://www.kdnuggets.com/2017/02/removing-outliers-standard-deviation-python.html
+    """
+    bands = bands or ee.Image(collection.first()).bandNames()
+    bands = ee.List(bands)
+    forstats = collection.select(bands)
+    mean = forstats.mean()
+    stddev = forstats.reduce(ee.Reducer.stdDev())
+    imin = mean.subtract(stddev.multiply(sigma))
+    imax = mean.add(stddev.multiply(sigma))
+
+    def getOutlier(im, imin, imax):
+        ismin = im.lt(imin)
+        ismax = im.gt(imax)
+        outlier = ismin.Or(ismax)
+        return outlier
+
+    def overcol(im):
+        outs = getOutlier(im.select(bands), imin, imax)
+        if updateMask:
+            ibands = im.select(bands)
+            ibands = ibands.updateMask(outs.Not())
+        else:
+            ibands = image_module.addSuffix(outs, '_outlier')
+
+        return im.addBands(ibands, overwrite=True)
+
+    return collection.map(overcol)
+
+
 def data2pandas(data):
     """
     Convert data coming from tools.imagecollection.get_values to a
